definition module ligretto
 
import iTasks

get_random_int :: Task Int

/* A model for the game of Ligretto.
*/
:: NrOfPlayers :== Int                            // 2 upto 4 players
:: Middle      :== [Pile]                         // 0 upto 16 piles
:: Pile        :== [Card]                         // 0 upto 10 cards of same front color
:: Card          = { back     :: Color            // the backside color (belonging to player)
                   , front    :: Color            // the frontside color (for playing)
                   , nr       :: Int              // 1 upto 10
                   }
:: SideUp        = Front | Back                   // the side of the card that faces upwards
:: Color         = Red | Green | Blue | Yellow    // the four player / card colors
:: Player        = { color     :: Color           // the backside color of this player's cards
                   , row       :: Row             // the row of the player (nr_of_cards_in_row nr_of_players)
                   , ligretto  :: Pile            // the ligretto pile: 10 upto 0 (win) cards
                   , hand      :: Hand            // the hand cards
                   }
:: Row         :== [Card]                         // nr_of_cards_in_row nr_of_players cards
:: Hand          = { conceal   :: Pile            // the concealed pile
                   , discard   :: Pile            // the discarded pile
                   }

/** nr_of_cards_in_row nr_of_players = n:
        @n is the number of cards that a row should have.
        @nr_of_players should be one of: 2, 3, 4.
*/
nr_of_cards_in_row   :: !NrOfPlayers -> Int

/** colors nr_of_players = player_colors:
        @player_colors are the colors in a game consisting of @nr_of_players players.
        @nr_of_players should be one of: 2, 3, 4.
*/
colors               :: !NrOfPlayers -> [Color]

/** initial_player nr_of_players color n = player:
        @player is a player with all cards shuffled and distributed to the @player.row, @player.ligretto, and @player.hand.
        @nr_of_players should be one of: 2, 3, 4.
        @color should be one of: (colors @nr_ofplayers).
*/
initial_player       :: !NrOfPlayers !Color !Int -> Player

/** pick_row_card row_nr player = (card, player`):
        @card is the card at @row_nr in the current @player.row (counted as 1..nr_of_players).
        This card is replaced in @player by the next card in @player.ligretto.
        @player.ligretto should not be empty.
*/
pick_row_card        :: !Int !Player -> (!Card,!Player)

/** top_discard player = Nothing:
        @player has no current cards in her @player.hand.discard pile.
    top_discard player = Just card:
        @player has @card on top of her @player.hand.discard pile.
*/
top_discard          :: !Player -> Maybe Card

/** shuffle_hand seed player = player`:
        shuffles the current @player.hand.discard pile and sets it to @player`.hand.conceal.
        @player.hand.conceal must not be empty.
*/
shuffle_hand         :: !Int !Player -> Player

/** pick_discard player = (card,player`):
        @card is the current top card in @player.hand.discard.
        In @player` this card is removed.
        @player.hand.discard must not be empty.
*/
pick_discard         :: !Player -> (!Card,!Player)

/** swap_discards player = player`:
        moves the top 3 cards from the @player.hand.conceal pile in reverse order to the @player.hand.discard pile.
        @player.hand.conceal must not be empty.
*/
swap_discards        :: !Player -> Player
